#include <string>
#include <vector>
using namespace std;

const unsigned int TAB_SPACE_COUNT = 4;
const int ROOT_INDENTATION = -42;

struct testcasePart
{
  unsigned int suppressed; // 0 if active, positive otherwise

  int totalIndentation; // used for trees

  string str1;
  string str2; // used for closing lines in trees

  vector<testcasePart*> children; // non-owning; used for children in trees
  testcasePart* parent;
  
  void treeOutput(ostream &out, bool yankMode, string newIndentation) const;

  testcasePart::testcasePart (const string &s) 
    : suppressed(0), totalIndentation(0), str1(s), parent(NULL) { }
};


class testcase
{
public:
  string prefix;
  string postfix;

  vector<testcasePart*> parts; // owning; used by flat algorithms. uses a pointer because they're created in multiple ways.
  testcasePart* treeRoot; // not an element of parts (special), but its children are. not sure this is the most sane way to do it.
  
  string naturalFilename;
  
  void readFromFile(const char *filename);
  void setParseModes(const string &parseInfo);

  void writeToDefault() const;
  void writeToFile(const char *filename) const;
  void writeToStream(ostream &out) const;
  // void show() const;

  enum { CHARS, LINES, TREE } parseMode;
  enum { NO_CLOSING_LINES, CLOSING_LINES_IFF_CHILDREN } treeParseMode; // meaningful iff in TREE mode.
  enum { YANK, DELSUBTREE } treeReductionMode; // meaningful iff in TREE mode.
  
  bool usesSpaces; // can i declare an initial value like this?
  bool usesTabs;
  
  testcase::testcase ()
    : treeRoot(NULL), parseMode(LINES), usesSpaces(false), usesTabs(false) { } 
    
  testcase::~testcase ();

private:
  // Related to parsing
  void lineAvailable(const string &line);
  void treeLineAvailable(const string &line);
  void start();
  void end();
  
  vector<testcasePart*> indentationStack;
};
